using System;
using System.Runtime.InteropServices;

namespace SpiDotNet
{

    class GpioCmT35 : IGpioInterface
    {
        private IntPtr hDrv = IntPtr.Zero;
        

        // Device CreateFile specific constants
        internal const Int32 INVALID_HANDLE_VALUE = -1;
       
        internal const uint GPIO_DIR_INPUT  =   (1 << 0);
        internal const uint GPIO_DIR_OUTPUT =   (0 << 0);
        internal const uint GPIO_INT_LOW_HIGH = (1 << 1);
        internal const uint GPIO_INT_HIGH_LOW = (1 << 2);
        internal const uint GPIO_INT_LOW     =  (1 << 3);
        internal const uint GPIO_INT_HIGH = (1 << 4);

        internal uint IOCTL_GPIO_SET_BIT = NativeMethods.CTL_CODE_DEVICE_UNKNOWN(0x0300);
        internal uint IOCTL_GPIO_CLR_BIT = NativeMethods.CTL_CODE_DEVICE_UNKNOWN(0x0301);
        internal uint IOCTL_GPIO_GETBIT  = NativeMethods.CTL_CODE_DEVICE_UNKNOWN(0x0303);
        internal uint IOCTL_GPIO_SETMODE = NativeMethods.CTL_CODE_DEVICE_UNKNOWN(0x0304);
        internal uint IOCTL_GPIO_GETMODE = NativeMethods.CTL_CODE_DEVICE_UNKNOWN(0x0305);
 
        [StructLayout(LayoutKind.Sequential)]
        internal class IoctlGpioInfo
        {
            public IoctlGpioInfo(uint gpio, uint direction)
            {
                this.num = gpio;
                this.dir = direction;
            }
            public uint num;
            public uint dir;
        }


        
        public GpioCmT35() 
        {
            //First, we create a handle to the I2C proxy driver. I2C3 in our case.
            hDrv = NativeMethods.CreateFile("GIO1:", 0, 0, IntPtr.Zero, 0, 0, IntPtr.Zero);
            if (hDrv.ToInt32() == INVALID_HANDLE_VALUE)
                throw new InvalidOperationException("Can't open GIO1:");
        }

        ~GpioCmT35()
        {
            NativeMethods.CloseHandle(hDrv);
        }

        public bool SetAsOutput(uint gpioNumber, GpioLevel level)
        {
            IoctlGpioInfo inputParameters = new IoctlGpioInfo(gpioNumber, GPIO_DIR_OUTPUT);

            bool res = NativeMethods.SendIOCTL(hDrv, IOCTL_GPIO_SETMODE, inputParameters, (uint)Marshal.SizeOf(inputParameters));
            
            if (!res)
                return false;

            if (level == GpioLevel.HIGH)
                return NativeMethods.SendIOCTL(hDrv, IOCTL_GPIO_SET_BIT,  gpioNumber, (uint)Marshal.SizeOf(gpioNumber));
            else
                return NativeMethods.SendIOCTL(hDrv, IOCTL_GPIO_CLR_BIT, gpioNumber, (uint)Marshal.SizeOf(gpioNumber));
        }

        public bool SetAsInput(uint gpioNumber)
        {
            IoctlGpioInfo inputParameters = new IoctlGpioInfo(gpioNumber, GPIO_DIR_INPUT);

            return NativeMethods.SendIOCTL(hDrv, IOCTL_GPIO_SETMODE, inputParameters, (uint)Marshal.SizeOf(inputParameters));
        }

        /**
        //	Gets the current configuration for the given GPIO
        */
        public GpioConfiguration GetConfiguration(uint gpioNumber)
        {
            UInt32 result = NativeMethods.GetIOCTL(hDrv, IOCTL_GPIO_GETMODE, gpioNumber);
            if ((result & GPIO_DIR_INPUT)!=0)
                return GpioConfiguration.INPUT;
            else
                return GpioConfiguration.OUTPUT;

        }

        /**
        // Get current level of an output gpio
        */
        public GpioLevel GetGpioLevel(uint gpioNumber)
        {
            UInt32 uiRes = NativeMethods.GetIOCTL(hDrv, IOCTL_GPIO_GETBIT, gpioNumber);
            if (uiRes != 0)
                return GpioLevel.HIGH;
            else
                return GpioLevel.LOW;
        }

    }
}